#############################################################################
#
# Copyright (c) 2009 Victorian Partnership for Advanced Computing Ltd and
# Contributors.
# All Rights Reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#############################################################################

from os import path
from M2Crypto import BIO, RSA, EVP, m2
from M2Crypto.util import no_passphrase_callback


class Getpget(object):
    """Class to allow m2crypto compat with the key class"""
    def __get__(self, obj, objtype):
        return obj._pubkey.pkey


class Key:
    """This is a wrapper class for handling key pair generation.

    :param key: the :class:`str` or file path to the key
    :param keySize: The size of the key to be generated (default 2048)
    :param callback: a function that is called when outputting the key, it's used to encrypt the key before writing it.

    """
    def __init__(self, key=None, keySize=2048, callback=no_passphrase_callback):
        self._passphrase_callback = callback
        if isinstance(key, str):
            key = key.strip()
            if key.startswith("-----BEGIN RSA PRIVATE KEY-----"):
                bio = BIO.MemoryBuffer(key)
                self._key = RSA.load_key_bio(bio, self._passphrase_callback)
            elif path.exists(key):
                keyfile = open(key)

                bio = BIO.File(keyfile)
                key = RSA.load_key_bio(bio, self._passphrase_callback)

                self._pubkey = EVP.PKey()
                self._key = key
                self._pubkey.assign_rsa(self._key)
            else:
                raise ValueError("WTF")
        else:
            self._pubkey = EVP.PKey()
            self._key = RSA.gen_key(keySize, m2.RSA_F4)
            self._pubkey.assign_rsa(self._key)


    pkey = Getpget()


    def __str__(self):
        bio = BIO.MemoryBuffer()
        self._key.save_key_bio(bio, cipher=None)
        return bio.read()




#############################################################################
#
# Copyright (c) 2009 Victorian Partnership for Advanced Computing Ltd and
# Contributors.
# All Rights Reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#############################################################################

import logging

from urllib import urlencode
import urllib2
import xml.dom.minidom

log = logging.getLogger('arcs.gsi')


class SLCSException(Exception):
    """Exception raised when an error is returned from the SLCS server.

    .. attribute:: expression input expression in which the error occurred
    .. attribute:: message explanation of the error
    """

    def __init__(self, expression, message):
        self.expression = expression
        self.message = message


def parse_req_response(response):
    """parse a SLCS response.

    Example SLCS Response::

        <?xml version="1.0" ?>
        <SLCSLoginResponse>
        <Status>Success</Status>
        <AuthorizationToken>EKJH54HJKRT908YSEDJNQ23QLIUYIU2HWQWDB12IEGB12DAIKJAAPQ1</AuthorizationToken>
        <CertificateRequest url="https://slcs1.arcs.org.au:443/SLCS/certificate">
        <Subject>DC=slcs,DC=org,DC=au,O=Org,CN=Common Name Dlw349sKRMWOsokaA</Subject>
        <CertificateExtension critical="false" name="ExtendedKeyUsage" oid="2.5.29.37">ClientAuth</CertificateExtension>
        <CertificateExtension critical="true" name="KeyUsage" oid="2.5.29.15">DigitalSignature,KeyEncipherment</CertificateExtension>
        <CertificateExtension critical="false" name="CertificatePolicies" oid="2.5.29.32">1.3.6.1.4.1.31863.1.0.1</CertificateExtension>
        <CertificateExtension critical="false" name="SubjectAltName" oid="2.5.29.17">email:user@host.localdomain</CertificateExtension>
        </CertificateRequest>
        </SLCSLoginResponse>

    :rtype: (token, dn, reqURL, extensions)

    """
    slcsRespDOM = xml.dom.minidom.parse(response)

    token = slcsRespDOM.getElementsByTagName("AuthorizationToken")[0]\
                       .childNodes[0].data
    dn = slcsRespDOM.getElementsByTagName("Subject")[0].childNodes[0].data
    reqURL = slcsRespDOM.getElementsByTagName("CertificateRequest")[0]\
                        .getAttribute('url')
    elements = []
    for e in slcsRespDOM.getElementsByTagName('CertificateExtension'):
        name = str(e.getAttribute('name'))
        critical = str(e.getAttribute('critical')) == 'true' or False
        value = str(e.childNodes[0].data)
        elements.append({'name':name, 'critical':critical, 'value':value})
    return token, dn, reqURL, elements


def parse_cert_response(response):
    """parse a SLCS signed certificate response and either return the certificate or None"""
    dom = xml.dom.minidom.parse(response)
    status = dom.getElementsByTagName("Status")[0].childNodes[0].data
    if status == 'Error':
        error = dom.getElementsByTagName("Error")[0].childNodes[0].data
        stack = dom.getElementsByTagName("StackTrace")[0].childNodes[0].data
        log.error(error)
        log.error(stack)
        raise SLCSException(error, stack)

    return ''.join([i.data for i in dom.getElementsByTagName("Certificate")[0].childNodes])

try:
    import arcs.gsi.certificate
except:
    pass


def slcs_handler(slcsResp):
    """
    Take a response from a SLCS Login URL and return the cert,keys

    :rtype: (key, pubkey, certifcate)
    """
    token, dn, reqURL, elements = parse_req_response(slcsResp)

    certreq = arcs.gsi.certificate.CertificateRequest(dn=str(dn), extensions=elements)
    certreq.sign()
    # POST the Token and CertReq back to the slcs server
    data = urlencode({'AuthorizationToken': token,
                      'CertificateSigningRequest': repr(certreq)})
    log.info('Request Signing by SLCS')
    log.debug('POST: %s' % reqURL)
    certResp = urllib2.urlopen(reqURL, data)
    cert = parse_cert_response(certResp)
    return arcs.gsi.certificate.Certificate(str(cert), certreq.get_key())





#############################################################################
#
# Copyright (c) 2009 Victorian Partnership for Advanced Computing Ltd and
# Contributors.
# All Rights Reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#############################################################################


from M2Crypto import X509
import datetime
import logging

log = logging.getLogger('arcs.gsi')

MBSTRING_ASC  = 0x1000 | 1

def _build_name_from_string(dn):
    """"
    Turns a DN from a string to a X509.X509_Name
    """
    x509Name = X509.X509_Name()
    splitchar = ','
    if dn.startswith('/'):
        splitchar = '/'
    for entry in dn.split(splitchar):
        l = entry.split("=")
        x509Name.add_entry_by_txt(field=str(l[0].strip()), type=MBSTRING_ASC,
                                      entry=str(l[1]),len=-1, loc=-1, set=0)
    return x509Name

def cert_time_diff(cert, date=None):
    """
    return the time until the certificate expires
    takes a certificate and an optional datetime object, default is Now
    """
    time = cert.get_times()[1]
    d1 = datetime.datetime.strptime(str(time), "%b %d %H:%M:%S %Y %Z")
    d2 = date or datetime.datetime.now()
    return d1 - d2




#############################################################################
#
# Copyright (c) 2009 Victorian Partnership for Advanced Computing Ltd and
# Contributors.
# All Rights Reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#############################################################################
from M2Crypto import X509, m2, BIO, ASN1, Err
from os import path
import logging
import time
from key import Key
from util import _build_name_from_string
from M2Crypto.util import no_passphrase_callback

log = logging.getLogger('arcs.gsi')

Att_map = {'extendedkeyusage': 'extendedKeyUsage',
           'keyusage': 'keyUsage',
           'certificatepolicies': 'certificatePolicies',
           'subjectaltname': 'subjectAltName',
           'proxycertinfo': 'proxyCertInfo',
          }

multi_attrs = {'keyusage': {'digitalsignature': 'Digital Signature',
                            'keyencipherment': 'Key Encipherment',
                            'dataencipherment': 'Data Encipherment',
                            }
               ,'extendedkeyusage': {'clientauth': 'clientAuth',
                                     }
              }


class CertificateRequest:
    """This is a wrapper class for handling certificate request generation.

    :param request: either a PEM :class:`str` a DER :class:`str`
    :param path: the path to the certificate request file
    :param key: if sepecified this will be wrapped in a :class:`~arcs.gsi.key.Key`
    :param keySize: The size of the key to be generated (default 2048)
    :param dn: the DN string of M2Crypto X509_Name
    :param extensions: a :class:`list` of :class:`dict` objects containing extensions

    """
    def __init__(self, request=None, path=None, dn=None,
                 keySize=2048, key=None, extensions=None):

        self.signed = False

        # Create public key object
        if key and not request:
            self._key = key
        else:
            self._key = Key(keySize=keySize)

        # Create certificate._request
        if request:
            self._request = request
            if isinstance(request, str):
                if request.startswith("-----BEGIN CERTIFICATE REQUEST-----"):
                    bio = BIO.MemoryBuffer(request)
                    cptr = m2.x509_req_read_pem(bio._ptr())
                    if cptr is None:
                        raise X509.X509Error(Err.get_error())
                    self._request = X509.Request(cptr, _pyfree=1)
                elif ord(request[0]) == 48:
                    bio = BIO.MemoryBuffer(request)
                    cptr = m2.d2i_x509_req(bio._ptr())
                    if cptr is None:
                        raise X509.X509Error(Err.get_error())
                    self._request = X509.Request(cptr, _pyfree=1)
                elif path.exists(request):
                    reqfile = open(request)
                    bio = BIO.File(reqfile)
                    self._request = X509.load_request_bio(bio)
                else:
                    raise ValueError('WFT')
        else:
            self._request = X509.Request()
            self._request.set_pubkey(self._key)
            self._request.set_version(0)

        if dn:
            self.set_dn(dn)

        if extensions:
            self.add_extensions(extensions)



    def set_dn(self, dn):
        """
        set the dn of the certificate request

        :param dn: either a string or an X509_Name object
        """

        if isinstance(dn, X509.X509_Name):
            self._request.set_subject_name(dn)
        elif isinstance(dn, str):
            self._request.set_subject_name(_build_name_from_string(dn))
        else:
            raise ValueError('WFT')
        self._signed = False


    def add_extensions(self, extensions):
        """
        add extenstions to the certificate. Takes a list of dictionary and
        converts them into extension objects.

        :param extensions: a :class:`list` of :class:`dict` objects containing extensions

        Example of adding an extension::

            >>> import arcs.gsi.certificate
            >>> r = arcs.gsi.certificate.CertificateRequest()
            >>> r.add_extensions([{'name':'subjectAltName' ,'critical':0, 'value':'russell@vpac.org'}])
        """
        extstack = X509.X509_Extension_Stack()

        sslower = lambda s: s.lower().replace(' ','')

        for e in extensions:
            name = e['name']
            key = sslower(name)
            critical = e['critical']
            if key in multi_attrs:
                e['value'] = ', '.join([multi_attrs[key][sslower(v)]
                               for v in e['value'].split(',')])
            extstack.push(X509.new_extension(Att_map[key],
                                             e['value'],
                                             critical=int(critical)))
        self._request.add_extensions(extstack)
        self._signed = False


    def sign(self, md='sha1'):
        """
        sign the certificate request, must be done to finiliaze the :class:`CertificateRequest`

        :param md: the hash to use when signing the :class:`CertificateRequest`
        """
        self._request.sign(self._key, md)
        self._signed = True


    def get_cert_req(self):
        """return the certificate request object"""
        if not self._signed:
            self.sign()
        return self._request

    def get_key(self):
        """return the private key pair"""
        return self._key

    def get_pubkey(self):
        """return the public key from the certificate request"""
        return self._request.get_pubkey()

    def as_dict(self):
        """return the dictionary representation of the certificate request"""
        c = {}
        c['version'] = self._request.get_version()
        c['subject'] = self._request.get_subject().as_text()
        # XXX Needs to return the extensions too.

    def __str__(self):
        return self._request.as_text()


    def __repr__(self):
        return self._request.as_pem()


class Certificate:
    def __init__(self, certificate=None, key=None,
                 callback=no_passphrase_callback):
        self._key = None
        if key:
            if isinstance(key, Key):
                self._key = key
            else:
                self._key = Key(key, callback=callback)

        if isinstance(certificate, str):
            if certificate.startswith("-----BEGIN CERTIFICATE-----"):
                self._certificate = X509.load_cert_string(str(certificate),
                                                          X509.FORMAT_PEM)
            elif path.exists(certificate):
                certfile = open(certificate)
                bio = BIO.File(certfile)
                self._certificate = X509.load_cert_bio(bio)
            else:
                raise ValueError("WTF")
        else:
            self._certificate = X509.X509()
            if not key:
                key = Key()
            self._key = key
            self.set_pubkey(self._key)


    def set_version(self, version):
        self._certificate.set_version(version)


    def get_dn(self):
        return self.get_subject().as_text()


    def set_dn(self, dn):
        if isinstance(dn, X509.X509_Name):
            self._certificate.set_subject_name(dn)
        elif isinstance(dn, str):
            self._certificate.set_subject_name(_build_name_from_string(dn))
        else:
            raise ValueError('WFT')


    def add_extensions(self, extensions):

        for e in extensions:
            self.add_extension(e)

        self._signed = False


    def add_extension(self, e):
        sslower = lambda s: s.lower().replace(' ','')
        name = e['name']
        key = sslower(name)
        critical = e['critical']
        if key in multi_attrs:
            e['value'] = ', '.join([multi_attrs[key][sslower(v)]
                           for v in e['value'].split(',')])
        self._certificate.add_ext(X509.new_extension(Att_map[key],
                                         e['value'],
                                         critical=int(critical)))


    def set_issuer_name(self, name):
        if isinstance(name, X509.X509_Name):
            self._certificate.set_issuer_name(name)
        elif isinstance(name, str):
            self._certificate.set_issuer_name(_build_name_from_string(name))
        else:
            raise ValueError('WFT')


    def set_times(self, lifetime=43200):
        """
        Sets the lifetime of the certificate
        Defaults to 12 hours
        """
        not_before = ASN1.ASN1_UTCTIME()
        not_after = ASN1.ASN1_UTCTIME()
        not_before.set_time(int(time.time()) - 300)
        not_after.set_time(int(time.time()) + lifetime )
        self._certificate.set_not_before(not_before)
        self._certificate.set_not_after(not_after)

    def get_times(self):
        """
        Return tuple containing not before and not after times
        """
        return (self._certificate.get_not_before(), self._certificate.get_not_after())


    def sign(self, key, md='sha1'):
        self._certificate.sign(key, md)


    def set_pubkey(self, pubkey):
        self._certificate.set_pubkey(pubkey)


    def get_ext(self, extension):
        """Get X509 extension by name"""
        return self._certificate.get_ext(extension)


    def get_serial_number(self):
        return self._certificate.get_serial_number()


    def get_issuer(self):
        return self._certificate.get_issuer()


    def get_version(self):
        return self._certificate.get_version()


    def get_subject(self):
        return self._certificate.get_subject()


    def get_pubkey(self):
        return self._certificate.get_pubkey()


    def get_key(self):
        if self._key:
            return self._key
        raise ValueError("No Key?")


    def __str__(self):
        return self._certificate.as_text()


    def __repr__(self):
        return self._certificate.as_pem()

    def as_der(self):
        return self._certificate.as_der()




#############################################################################
#
# Copyright (c) 2009 Victorian Partnership for Advanced Computing Ltd and
# Contributors.
# All Rights Reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#############################################################################

__docformat__ = 'restructuredtext'

import os, sys

import unittest
import doctest

def setUp(test):
    import arcs.gsi.certificate

def tearDown(test):
    pass


current_dir = os.path.dirname(__file__) + '/../../../../../../'


def doc_suite(test_dir, setUp=setUp, tearDown=tearDown, globs=None):
    """Returns a test suite, based on doctests found in /docs."""
    suite = []
    if globs is None:
        globs = globals()

    globs['test_dir'] = current_dir

    flags = (doctest.ELLIPSIS | doctest.NORMALIZE_WHITESPACE |
             doctest.REPORT_ONLY_FIRST_FAILURE)

    package_dir = os.path.split(test_dir)[0]
    if package_dir not in sys.path:
        sys.path.append(package_dir)

    docs = []
    for dir_ in ('docs',):
        doctest_dir = os.path.join(package_dir, dir_)

        # filtering files on extension
        docs.extend([os.path.join(doctest_dir, doc) for doc in
                     os.listdir(doctest_dir) if doc.endswith('.rst')])
    for test in docs:
        suite.append(doctest.DocFileSuite(test, optionflags=flags,
                                          globs=globs, setUp=setUp,
                                          tearDown=tearDown,
                                          module_relative=False))

    return unittest.TestSuite(suite)

def test_suite():
    """returns the test suite"""
    return doc_suite(current_dir)

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



#############################################################################
#
# Copyright (c) 2009 Victorian Partnership for Advanced Computing Ltd and
# Contributors.
# All Rights Reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#############################################################################

from M2Crypto import EVP, m2, X509
import struct

from certificate import Certificate

MBSTRING_ASC  = 0x1000 | 1
PCI_VALUE_FULL = "critical, language:Inherit all"
PCI_VALUE_LIMITED = "critical, language:1.3.6.1.4.1.3536.1.1.1.9"


class ProxyCertificate:
    """This is a wrapper class for handling proxy certificate generation.

    :param request: if specified this will be wrapped in a :class:`~arcs.gsi.certificate.Certificate` object
    :param proxykey: the key used to sign the proxy, if sepecified this will be wrapped in a :class:`~arcs.gsi.key.Key` object
    :param full: whether this is a full proxy or not

    """
    def __init__(self, certificate, proxykey=None, full=True):
        if isinstance(certificate, Certificate):
            self._certificate = certificate
        elif isinstance(certificate, ProxyCertificate):
            self._certificate = certificate._proxy
        else:
            self._certificate = Certificate(certificate)
        self._full = full

        self._proxy = Certificate(key=proxykey)

        self._proxy.set_version(2)
        self.set_serial_number()
        issuer = self._certificate.get_subject()
        subject = X509.X509_Name()
        for n in issuer:
            m2.x509_name_add_entry(subject.x509_name, n.x509_name_entry, -1, 0)
        subject.add_entry_by_txt(field='CN', type=MBSTRING_ASC,
                                 entry=str(self._proxy.get_serial_number()),len=-1, loc=-1, set=0)
        self._proxy.set_dn(subject)
        self._proxy.set_times(lifetime=43200)
        self._proxy.set_issuer_name(self._certificate.get_subject())

        try:
            key_usage = self._certificate._certificate.get_ext('keyUsage')
        except LookupError:
            pass
        else:
            self._proxy.add_extension({'name':key_usage.get_name(),
                                       'critical': key_usage.get_critical(),
                                       'value': self._fix_key_usage(key_usage.get_value())})

        if self._full:
            self._proxy.add_extension({'name':"proxyCertInfo",
                                       'value': PCI_VALUE_FULL,
                                       'critical': 1})
        else:
            self._proxy.add_extension({'name': "proxyCertInfo",
                                           'value': PCI_VALUE_LIMITED,
                                           'critical': 1})

    def _fix_key_usage(self, values):
        """
        invalid values are 'Non Repudiation' and 'keyCertSign'
        digitalSignature is required
        """
        r = []
        for v in values.split(', '):
            if v in ['Non Repudiation', 'keyCertSign']:
                continue
            r.append(v)
        if 'Digital Signature' not in r:
            r.append('Digital Signature')
        return ', '.join(r)


    def set_serial_number(self):
        message_digest = EVP.MessageDigest('sha1')
        pubkey = self._proxy.get_pubkey()
        der_encoding = pubkey.as_der()
        message_digest.update(der_encoding)
        digest = message_digest.final()
        digest_tuple = struct.unpack('BBBB', digest[:4])
        sub_hash = long(digest_tuple[0] + (digest_tuple[1] + ( digest_tuple[2] +
                               ( digest_tuple[3] >> 1) * 256 ) * 256) * 256)
        self._proxy._certificate.set_serial_number(sub_hash)


    def sign(self, md='sha1'):
        self._proxy.sign(self._certificate.get_key(), md)


    def __str__(self):
        return self._proxy.__str__()


    def __repr__(self):
        return self._proxy.__repr__()

    def as_der(self):
        return self._proxy._certificate.as_der()





from setuptools import setup, find_packages
from xml.dom.minidom import parse
import os

ex_req = []
# a hack because redhat setuptools can't detect m2crypto
if not os.path.exists('/etc/redhat-release'):
    ex_req.append('M2Crypto')

# Get version from common file
pom = parse('pom.xml')
for t in pom.getElementsByTagName('project')[0].childNodes:
    if t.nodeName == 'version':
        version = t.childNodes[0].nodeValue.rstrip('-SNAPSHOT')
        break

setup(name='arcs.gsi',
      version=version,
      description="Library to assist GSI authentication and certificate handling in python.",
      long_description=".. contents::\n\n" +
                       open(os.path.join("docs", "introduction.rst")).read() + "\n" +
                       open(os.path.join("docs", "changes.rst")).read(),
      # Get more strings from http://www.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "License :: OSI Approved :: GNU General Public License (GPL)",
        "Topic :: Security :: Cryptography",
        "Intended Audience :: Developers",
        "Topic :: System :: Distributed Computing",
        ],
      keywords='',
      author='Russell Sim',
      author_email='russell.sim@arcs.org.au',
      url='http://code.arcs.org.au/gitorious/arcs-gsi/arcs-gsi',
      download_url='http://code.arcs.org.au/pypi/simple/arcs.gsi/',
      license='GPL',
      packages=find_packages('src/main/python', exclude=['ez_setup']),
      package_dir = {'': 'src/main/python'},
      namespace_packages=['arcs'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          # -*- Extra requirements: -*-
      ] + ex_req,
      entry_points="""
      # -*- Entry points: -*-
      [console_scripts]
      proxy-init = arcs.gsi.proxyinit:main
      """,
      )
